"""
Service base specification module.
"""
from __future__ import annotations

from digitalhub_core.entities._base.spec import Spec, SpecParams


class ServiceSpec(Spec):
    """
    Service specifications.
    """

    def __init__(self) -> None:
        """
        Constructor.
        """


class ServiceParams(SpecParams):
    """
    Service parameters.
    """
